@@ -2,7 +2,7 @@
    errs.c
 
    This file is part of GNU Anubis.
-   Copyright (C) 2001, 2002 The Anubis Team.
+   Copyright (C) 2001, 2002, 2003 The Anubis Team.
 
    GNU Anubis is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -32,12 +32,7 @@ anubis_error(int method, char *format, ...)
 	char txt[LINEBUFFER+1];
 
 	va_start(arglist, format);
-	#ifdef HAVE_VSNPRINTF
-	vsnprintf(txt, LINEBUFFER,
-	#else
-	vsprintf(txt,
-	#endif /* HAVE_VSNPRINTF */
-		format, arglist);
+	vsnprintf(txt, LINEBUFFER, format, arglist);
 	va_end(arglist);
 
 	if (method == HARD)
@@ -46,7 +41,7 @@ anubis_error(int method, char *format, ...)
 		topt &= ~T_ERROR;
 
 	if (options.termlevel != SILENT) {
-		#ifdef HAVE_SYSLOG
+#ifdef HAVE_SYSLOG
 		if ((topt & T_DAEMON) && !(topt & T_FOREGROUND)) {
 			if (options.slogfile)
 				filelog(options.slogfile, txt);
@@ -57,7 +52,7 @@ anubis_error(int method, char *format, ...)
 				filelog(options.ulogfile, txt);
 		}
 		else
-		#endif /* HAVE_SYSLOG */
+#endif /* HAVE_SYSLOG */
 			if (topt & T_FOREGROUND)
 				mprintf(">>[%d] %s", (int)getpid(), txt);
 			else
@@ -70,9 +65,10 @@ anubis_error(int method, char *format, ...)
 }
 
 void
-socket_error(void)
+socket_error(const char *msg)
 {
-	anubis_error(HARD, _("Couldn't write on socket! %s."), strerror(errno));
+	anubis_error(HARD, _("Couldn't write to socket: %s."),
+		     msg ? msg : strerror(errno));
 	return;
 }
 
