@@ -2,7 +2,7 @@
    exec.c
 
    This file is part of GNU Anubis.
-   Copyright (C) 2001, 2002 The Anubis Team.
+   Copyright (C) 2001, 2002, 2003 The Anubis Team.
 
    GNU Anubis is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -72,7 +72,7 @@ gen_execargs(const char *commandline)
 static int
 make_sockets(int fd[2])
 {
-	#ifndef HAVE_SOCKETPAIR
+#ifndef HAVE_SOCKETPAIR
 	struct sockaddr_in addr;
 	int addrlen;
 	int sd;
@@ -115,12 +115,12 @@ make_sockets(int fd[2])
 		return -1;
 	}
 	close_socket(sd);
-	#else
+#else
 	if (socketpair(AF_UNIX, SOCK_STREAM, 0, fd)) {
 		anubis_error(HARD, _("socketpair() failed: %s."), strerror(errno));
 		return -1;
 	}
-	#endif /* not HAVE_SOCKETPAIR */
+#endif /* not HAVE_SOCKETPAIR */
 	return 0;
 }
 
@@ -142,7 +142,7 @@ make_local_connection(char *exec_path, char **exec_args)
 	char **pargs;
 	char args[LINEBUFFER+1];
 
-	if (check_filename(exec_path) == 0)
+	if (check_filename(exec_path, 0) == 0)
 		return -1;
 
 	memset(args, 0, LINEBUFFER + 1);
@@ -176,9 +176,9 @@ make_local_connection(char *exec_path, char **exec_args)
 			return -1;
 	}
 	close(fd[1]);
-	#ifdef FD_CLOEXEC
+#ifdef FD_CLOEXEC
 	fcntl(fd[0], F_SETFD, FD_CLOEXEC);
-	#endif /* FD_CLOEXEC */
+#endif /* FD_CLOEXEC */
 	return fd[0];
 }
 
@@ -190,11 +190,8 @@ make_local_connection(char *exec_path, char **exec_args)
 char *
 external_program(int *rs, char *path, char *src, char *dst, int dstsize)
 {
-	int status;
-	int fd;
-	int n;
+	char *ret;
 	char tmp[LINEBUFFER+1];
-	char *buf = 0;
 	char **args = 0;
 	char *a = 0; /* args */
 	char *p = 0; /* path */
@@ -207,12 +204,7 @@ external_program(int *rs, char *path, char *src, char *dst, int dstsize)
 			p++;
 		else
 			p = path;
-		#ifdef HAVE_SNPRINTF
-		snprintf(tmp, LINEBUFFER,
-		#else
-		sprintf(tmp,
-		#endif /* HAVE_SNPRINTF */
-			"%s %s", p, a);
+		snprintf(tmp, LINEBUFFER, "%s %s", p, a);
 		p = path;
 		a = tmp;
 	}
@@ -224,10 +216,22 @@ external_program(int *rs, char *path, char *src, char *dst, int dstsize)
 		else
 			a = path;
 	}
-
+	
 	args = gen_execargs(a);
-	fd = make_local_connection(p, args);
+	ret = exec_argv(rs, args, src, dst, dstsize);
 	xfree_pptr(args);
+	return ret;
+}
+
+char *
+exec_argv(int *rs, char **argv, char *src, char *dst, int dstsize)
+{
+	int status;
+	int fd;
+	int n;
+	char *buf;
+	
+	fd = make_local_connection(argv[0], argv);
 	if (fd == -1) {
 		*rs = -1;
 		return 0;
@@ -246,20 +250,17 @@ external_program(int *rs, char *path, char *src, char *dst, int dstsize)
 	memset(dst, 0, dstsize);
 
 	if (dst && dstsize) { /* static array */
-		while ((n = read(fd, buf, DATABUFFER)) > 0)
-		{
+		while ((n = read(fd, buf, DATABUFFER)) > 0) {
 			strncat(dst, buf, dstsize);
 			memset(buf, 0, DATABUFFER + 1);
 			dstsize -= n;
 			if (dstsize < 1)
 				break;
 		}
-	}
-	else { /* dynamic array */
+	} else { /* dynamic array */
 		dst = (char *)xmalloc(1);
-		while ((n = read(fd, buf, DATABUFFER)) > 0)
-		{
-			dst = (char *)xrealloc((char *)dst, strlen(dst) + n + 1);
+		while ((n = read(fd, buf, DATABUFFER)) > 0) {
+			dst = xrealloc(dst, strlen(dst) + n + 1);
 			strncat(dst, buf, n);
 			memset(buf, 0, DATABUFFER + 1);
 		}
