@@ -2,7 +2,7 @@
    mem.h
 
    This file is part of GNU Anubis.
-   Copyright (C) 2001, 2002 The Anubis Team.
+   Copyright (C) 2001, 2002, 2003 The Anubis Team.
 
    GNU Anubis is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -26,17 +26,19 @@
    Free unused memory. Global pointers.
 */
 
-#define xfree(p) \
+#define xfree(p) do\
 	if (p) { \
 		free(p); \
-		p = 0; \
-	}
+		p = NULL; \
+	}\
+     while (0)
 
-#define xfree_pptr(p) \
+#define xfree_pptr(p) do\
 	if (p) { \
 		free_pptr(p); \
-		p = 0; \
-	}
+		p = NULL; \
+	}\
+     while (0)
 
 /* EOF */
 
