@@ -2,7 +2,7 @@
    misc.c
 
    This file is part of GNU Anubis.
-   Copyright (C) 2001, 2002 The Anubis Team.
+   Copyright (C) 2001, 2002, 2003 The Anubis Team.
 
    GNU Anubis is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -25,51 +25,74 @@
 #include "headers.h"
 #include "extern.h"
 
-struct list *
-new_element(struct list *p, struct list **head, char *newline)
+/* String lists */
+
+static int
+_mem_free(void *item, void *data)
 {
-	struct list *new;
-	new = (struct list *)xmalloc(sizeof(struct list));
-
-	if (*head == NULL)
-		p = NULL;
-
-	if (p) {
-		p->next = new;
-		new->next = NULL;
-		new->modify = NULL;
-		new->line = strdup(newline);
-	}
-	else {
-		new->next = *head;
-		new->line = strdup(newline);
-		new->modify = NULL;
-		*head = new;
-	}
-
-	return (struct list *)new;
+	free(item);
+	return 0;
 }
 
 void
-destroy_list(struct list **head)
+destroy_string_list(LIST **plist)
 {
-	struct list *p1 = *head;
-	struct list *p2 = NULL;
+	list_destroy(plist, _mem_free, NULL);
+}
 
-	if (*head == NULL)
-		return;
-	do {
-		p2 = p1->next;
-		free(p1->line);
-		if (p1->modify)
-			free(p1->modify);
-		free(p1);
-		if (p2)
-			p1 = p2;
-	} while (p2 != NULL);
-	*head = NULL;
+static int
+_assoc_free(void *item, void *data)
+{
+	assoc_free(item);
+	return 0;
+}
 
-	return;
+void
+destroy_assoc_list(LIST **plist)
+{
+	list_destroy(plist, _assoc_free, NULL);
+}
+
+void
+assoc_free(ASSOC *asc)
+{
+	free(asc->key);
+	free(asc->value);
+	free(asc);
+}
+
+ASSOC *
+header_assoc(char *line)
+{
+	char *p = strchr(line, ':');
+	ASSOC *entry = xmalloc(sizeof(*entry));
+	if (p) {
+		int len = p - line;
+		entry->key = xmalloc(len + 1);
+		memcpy(entry->key, line, len);
+		entry->key[len] = 0;
+		for (p++; *p && isspace(*p); p++)
+			;
+		entry->value = strdup(p);
+	} else {
+		/* Malformed header. Save everything as rhs */
+		entry->key = NULL;
+		entry->value = strdup(line);
+	}
+	return entry;
+}
+
+char *
+assoc_to_header(ASSOC *asc)
+{
+	char *buf;
+
+	if (asc->key) {
+		buf = xmalloc(strlen(asc->key) + strlen(asc->value) + 3);
+		sprintf(buf, "%s: %s", asc->key, asc->value);
+	} else
+		buf = strdup(asc->value);
+	return buf;
 }
 
 /****************************
@@ -178,41 +201,7 @@ remcrlf(char *s)
  Substitutions (RE back-references)
 ************************************/
 
-char *
-substitute(char *inbuf, char **subbuf)
-{
-	char **tmp = subbuf;
-	char *tmpout = 0;
-	char *tmpbuf = 0;
-	char sign[5];
-	int i = 0;
-
-	if (inbuf == 0 || subbuf == 0)
-		return 0;
-
-	tmpbuf = allocbuf(inbuf, 0);
-	tmp++;
-	while (*tmp)
-	{
-		#ifdef HAVE_SNPRINTF
-		snprintf(sign, 4,
-		#else
-		sprintf(sign,
-		#endif /* HAVE_SNPRINTF */
-			"\\%d", i + 1);
-		tmpout = insert(tmpbuf, sign, *tmp);
-		if (tmpout) {
-			tmpbuf = (char *)xrealloc((char *)tmpbuf, strlen(tmpout) + 1);
-			strcpy(tmpbuf, tmpout);
-			free(tmpout);
-		}
-		tmp++;
-		i++;
-	}
-	return tmpbuf;
-}
-
-char *
+static char *
 insert(char *inbuf, char *sign, char *fill_in)
 {
 	int len1 = 0;
@@ -253,8 +242,37 @@ insert(char *inbuf, char *sign, char *fill_in)
 	return outbuf;
 }
 
+char *
+substitute(char *inbuf, char **subbuf)
+{
+	char **tmp = subbuf;
+	char *tmpout = 0;
+	char *tmpbuf = 0;
+	char sign[5];
+	int i = 0;
+
+	if (inbuf == 0 || subbuf == 0)
+		return 0;
+
+	tmpbuf = allocbuf(inbuf, 0);
+	tmp++;
+	while (*tmp)
+	{
+		snprintf(sign, 4, "\\%d", i + 1);
+		tmpout = insert(tmpbuf, sign, *tmp);
+		if (tmpout) {
+			tmpbuf = (char *)xrealloc((char *)tmpbuf, strlen(tmpout) + 1);
+			strcpy(tmpbuf, tmpout);
+			free(tmpout);
+		}
+		tmp++;
+		i++;
+	}
+	return tmpbuf;
+}
+
 /***************************
- change to lower characters
+ Change to lower characters
 ****************************/
 
 void
@@ -272,25 +290,5 @@ change_to_lower(char *s)
 	return;
 }
 
-/****************
- ROT-13 encoding
-*****************/
-
-void
-check_rot13(void)
-{
-	if (message.body && (mopt & M_ROT13B)) {
-		int c;
-		char *txt = message.body;
-		for (c = strlen(txt) - 1; c >= 0; c--)
-		{
-			txt[c] = (islower((unsigned char)txt[c])
-			? 'a'+ (txt[c] - 'a' + 13)%26 : isupper((unsigned char)txt[c])
-			? 'A' + (txt[c] - 'A' + 13)%26 : txt[c]);
-		}
-	}
-	return;
-}
-
 /* EOF */
 
