@@ -2,7 +2,7 @@
    net.c
 
    This file is part of GNU Anubis.
-   Copyright (C) 2001, 2002 The Anubis Team.
+   Copyright (C) 2001, 2002, 2003 The Anubis Team.
 
    GNU Anubis is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -28,6 +28,64 @@
 static int connect_directly_to(char *, unsigned int);
 static int mread(int, void *, char *);
 
+static struct _debug_cache {
+	int method;
+	int output;
+	int newline;
+	size_t count;
+} debug_cache = { -1, -1, 0, 0 };
+
+static void
+_debug_printer(int method, int output, unsigned long nleft, char *ptr)
+{
+	int i;
+	char *mode = "?";
+
+	switch (method) {
+	case CLIENT:
+		mode = _("SERVER");
+		break;
+	case SERVER:
+		mode = _("CLIENT");
+	}
+
+	if (debug_cache.newline
+	    || method != debug_cache.method || output != debug_cache.output) {
+		if (!debug_cache.newline && debug_cache.count)
+			fprintf(stderr, "(%lu)\n",
+				(unsigned long)debug_cache.count);
+		debug_cache.method = method;
+		debug_cache.output = output;
+		debug_cache.newline = 0;
+		debug_cache.count = 0;
+		fprintf(stderr, "%s %s ", mode, output ? ">>>" : "<<<");
+	}
+
+	for (i = 0; i < nleft; i++, ptr++) {
+		debug_cache.count++;
+		debug_cache.newline = 0;
+		if (*ptr == '\r')
+			continue;
+		if (*ptr == '\n') {
+			fprintf(stderr, "(%ld)\n",
+				(unsigned long)debug_cache.count);
+			debug_cache.count = 0;
+			if (i != nleft-1) {
+				fprintf(stderr, "%s %s ",
+					mode, output ? ">>>" : "<<<");
+				debug_cache.newline = 0;
+			} else
+				debug_cache.newline = 1;
+		} else 
+			fputc(*ptr, stderr);
+	}
+}
+
+#define DPRINTF(method, output, nleft, ptr) do {\
+  if (options.termlevel == DEBUG) \
+     _debug_printer(method, output, nleft, ptr);\
+  } while (0)
+
 int
 make_remote_connection(char *host, unsigned int port)
 {
@@ -122,7 +180,8 @@ bind_and_listen(char *host, unsigned int port)
 	int sd = 0;
 	unsigned long inaddr;
 	struct sockaddr_in addr;
-
+	int true = 1;
+	
 	memset(&addr, 0, sizeof(addr));
 	addr.sin_family = AF_INET;
 	addr.sin_port = htons(port);
@@ -153,6 +212,8 @@ bind_and_listen(char *host, unsigned int port)
 	else
 		addr.sin_addr.s_addr = htonl(INADDR_ANY);
 
+	setsockopt (sd, SOL_SOCKET, SO_REUSEADDR, &true, sizeof(true));
+	
 	if (bind(sd, (struct sockaddr *)&addr, sizeof(addr)))
 		anubis_error(HARD, _("bind() failed: %s."), strerror(errno));
 	info(VERBOSE, _("GNU Anubis bound to %s:%u"), inet_ntoa(addr.sin_addr),
@@ -162,6 +223,73 @@ bind_and_listen(char *host, unsigned int port)
 	return sd;
 }
 
+static const char *
+_def_strerror(int rc)
+{
+	return strerror(rc);
+}
+
+static int
+_def_write(void *sd, char *data, size_t size, size_t *nbytes)
+{
+	int rc = send((int)sd, data, size, 0);
+	if (rc >= 0) {
+		*nbytes = rc;
+		return 0;
+	}
+	return errno;
+}
+
+static int
+_def_read(void *sd, char *data, size_t size, size_t *nbytes)
+{
+	int rc = recv((int)sd, data, size, 0);
+	if (rc >= 0) {
+		*nbytes = rc;
+		return 0;
+	}
+	return errno;
+}
+
+static int
+_def_close(void *sd_unused)
+{
+	return 0;
+}
+
+struct io_data {
+	net_io_t read;
+	net_io_t write;
+	strerror_t strerror;
+	net_close_t close;
+};
+
+struct io_data io_data[2] = {
+	/* CLIENT */
+	{ _def_read, _def_write, _def_strerror, _def_close },
+	/* SERVER */
+	{ _def_read, _def_write, _def_strerror, _def_close }
+};
+
+void
+net_set_io(int method, net_io_t read, net_io_t write,
+	   net_close_t close, strerror_t strerror)
+{
+	io_data[method].read = read ? read : _def_read;
+	io_data[method].write = write ? write : _def_write;
+	io_data[method].close = close;
+	io_data[method].strerror = strerror ? strerror : _def_strerror;
+}
+
+void
+net_close(int method, void *sd)
+{
+	if (io_data[method].close) {
+		io_data[method].close(sd);
+		net_set_io(method, NULL, NULL, NULL, NULL);
+	}
+}
+
 /**************
   Send a data
 ***************/
@@ -169,85 +297,20 @@ bind_and_listen(char *host, unsigned int port)
 void
 swrite(int method, void *sd, char *ptr)
 {
-	unsigned long nleft;
-	unsigned long nwritten = 0;
+	int rc;
+	size_t nleft, nwritten = 0;
 
 	if (ptr == 0)
 		return;
 
 	nleft = (unsigned long)strlen(ptr);
-	while (nleft > 0)
-	{
-		if (method == CLIENT) {
-
-			#ifdef HAVE_TLS
-			if (topt & T_SSL_CLIENT) {
-				if ((nwritten = (unsigned long)gnutls_record_send(
-						(gnutls_session)sd, ptr, nleft)) <= 0) {
-					socket_error();
-					return;
-				}
-			} else
-			#endif /* HAVE_TLS */
-
-			#ifdef HAVE_SSL
-			if (topt & T_SSL_CLIENT) {
-				if ((nwritten = (unsigned long)SSL_write((SSL *)sd,
-						ptr, nleft)) <= 0) {
-					socket_error();
-					return;
-				}
-			} else
-			#endif /* HAVE_SSL */
-
-			if (!(topt & T_SSL_CLIENT)) {
-				if ((nwritten = (unsigned long)send((int)sd,
-						ptr, nleft, 0)) == -1) {
-					socket_error();
-					return;
-				}
-			}
-			if (options.termlevel == DEBUG)
-				fprintf(stderr, _("(%ld)SERVER >>> %s"), nleft, ptr);
-		}
-		else if (method == SERVER) {
-
-			#ifdef HAVE_TLS
-			if (topt & T_SSL_SERVER) {
-				if ((nwritten = (unsigned long)gnutls_record_send(
-						(gnutls_session)sd, ptr, nleft)) <= 0) {
-					socket_error();
-					return;
-				}
-			} else
-			#endif /* HAVE_TLS */
-
-			#ifdef HAVE_SSL
-			if (topt & T_SSL_SERVER) {
-				if ((nwritten = (unsigned long)SSL_write((SSL *)sd,
-						ptr, nleft)) <= 0) {
-					socket_error();
-					return;
-				}
-			} else
-			#endif /* HAVE_SSL */
-
-			if ((int)sd == -1) { /* standard output */
-				nwritten = (unsigned long)write(1, ptr, nleft);
-			}
-			else if (!(topt & T_SSL_SERVER)) {
-				if ((nwritten = (unsigned long)send((int)sd,
-						ptr, nleft, 0)) == -1) {
-					socket_error();
-					return;
-				}
-			}
-			if (options.termlevel == DEBUG)
-				fprintf(stderr, _("(%ld)CLIENT >>> %s"), nleft, ptr);
-		}
-		if (nwritten <= 0)
+	while (nleft > 0) {
+		rc = io_data[method].write(sd, ptr, nleft, &nwritten);
+		if (rc) {
+			socket_error(io_data[method].strerror(rc));
 			return;
-
+		}
+		DPRINTF(method, 1, nwritten, ptr);
 		nleft -= nwritten;
 		ptr   += nwritten;
 	}
@@ -261,52 +324,17 @@ swrite(int method, void *sd, char *ptr)
 static int
 mread(int method, void *sd, char *ptr)
 {
-	static int nread = 0;
+	static size_t nread = 0;
 	static char *read_ptr = 0;
 	static char buf[LINEBUFFER+1];
 
 	if (nread <= 0) {
-	again:
-		if (method == CLIENT) {
-
-			#ifdef HAVE_TLS
-			if (topt & T_SSL_CLIENT)
-				nread = gnutls_record_recv((gnutls_session)sd, buf, LINEBUFFER);
-			else
-			#endif /* HAVE_TLS */
-
-			#ifdef HAVE_SSL
-			if (topt & T_SSL_CLIENT)
-				nread = SSL_read((SSL *)sd, buf, LINEBUFFER);
-			else
-			#endif /* HAVE_SSL */
-
-			if (!(topt & T_SSL_CLIENT))
-				nread = recv((int)sd, buf, LINEBUFFER, 0);
-		}
-		else if (method == SERVER) {
-
-			#ifdef HAVE_TLS
-			if (topt & T_SSL_SERVER)
-				nread = gnutls_record_recv((gnutls_session)sd, buf, LINEBUFFER);
-			else
-			#endif /* HAVE_TLS */
-
-			#ifdef HAVE_SSL
-			if (topt & T_SSL_SERVER)
-				nread = SSL_read((SSL *)sd, buf, LINEBUFFER);
-			else
-			#endif /* HAVE_SSL */
-
-			if (!(topt & T_SSL_SERVER))
-				nread = recv((int)sd, buf, LINEBUFFER, 0);
-		}
-		if (nread < 0) {
-			if (errno == EINTR)
-				goto again;
+		int rc = io_data[method].read(sd, buf, LINEBUFFER, &nread);
+		if (rc) {
+			socket_error(io_data[method].strerror(rc));
 			return -1;
 		}
-		else if (nread == 0)
+		if (nread == 0)
 			return 0;
 		read_ptr = buf;
 	}
@@ -318,55 +346,29 @@ mread(int method, void *sd, char *ptr)
 int
 recvline(int method, void *sd, void *vptr, int maxlen)
 {
-	int n, rc;
+	int n, rc, addc = 0;
 	char c, *ptr;
 
-	if ((int)sd == -1 && method == SERVER) { /* standard input */
-		memset(vptr, 0, maxlen);
-		#ifdef HAVE_ISATTY
-		if (!isatty(fileno(stdin))) {
-			fgets((char *)vptr, maxlen, stdin);
-			remcrlf((char *)vptr);
-			strcat((char *)vptr, CRLF);
-			if (vptr)
-				n = strlen((char *)vptr);
-			else
-				n = 0;
-		}
-		else
-		#endif /* HAVE_ISATTY */
-		{
-			n = read(0, (char *)vptr, maxlen - 1);
-			remcrlf((char *)vptr);
-			strcat((char *)vptr, CRLF);
-		}
-		return n;
-	}
-
 	ptr = vptr;
-	for (n = 1; n < maxlen; n++)
-	{
+	for (n = 1; n < maxlen; n++) {
 		if ((rc = mread(method, sd, &c)) == 1) {
+			if (c == '\n' && n > 1 && ptr[-1] != '\r') {
+				addc++;
+				*ptr++ = '\r';
+			}
 			*ptr++ = c;
-			if (c == '\n')
+			if (c == '\n') 
 				break;
-		}
-		else if (rc == 0) {
+		} else if (rc == 0) {
 			if (n == 1)
 				return 0;
 			else
 				break;
-		}
-		else
+		} else
 			return -1;
 	}
 	*ptr = 0;
-	if (options.termlevel == DEBUG) {
-		if (method == CLIENT)
-			fprintf(stderr, _("(%d)SERVER <<< %s"), n, (char *)vptr);
-		else if (method == SERVER)
-			fprintf(stderr, _("(%d)CLIENT <<< %s"), n, (char *)vptr);
-	}
+	DPRINTF(method, 0, n + addc, (char *)vptr);
 	return n;
 }
 
@@ -404,5 +406,6 @@ close_socket(int sd)
 	return;
 }
 
+
 /* EOF */
 
