@@ -2,7 +2,7 @@
    quit.c
 
    This file is part of GNU Anubis.
-   Copyright (C) 2001, 2002 The Anubis Team.
+   Copyright (C) 2001, 2002, 2003 The Anubis Team.
 
    GNU Anubis is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -42,40 +42,20 @@ sig_timeout(int code)
 void
 free_mem(void)
 {
-	xfree(message.body);
-	xfree(message.body_append);
-	xfree(message.boundary);
-	xfree(rm.rrt);
-	xfree(rm.post);
-	xfree(rm.header);
-	xfree(rm.gpg_file);
-	xfree(rm.latent_time);
-
-	#if defined(HAVE_TLS) || defined(HAVE_SSL)
+#ifdef USE_SSL
 	xfree(secure.cafile);
 	xfree(secure.cert);
 	xfree(secure.key);
-	#endif /* HAVE_TLS or HAVE_SSL */
+#endif /* HAVE_TLS or HAVE_SSL */
 
-	#ifdef HAVE_GPG
-	xfree(gpg.keys);
-	xfree(gpg.rm_key);
-	if (gpg.passphrase) {
-		memset(gpg.passphrase, 0, strlen(gpg.passphrase));
-		xfree(gpg.passphrase);
-	}
-	#endif /* HAVE_GPG */
+#ifdef HAVE_GPG
+	gpg_free();
+#endif /* HAVE_GPG */
 
-	destroy_list(&session.transmap);
 	xfree(options.slogfile);
 	xfree(options.ulogfile);
 	xfree(session.execpath);
-	if (topt & T_RCEXECARGS) {
-		xfree_pptr(session.execargs);
-		topt &= ~T_RCEXECARGS;
-	}
-	xfree_pptr(submatch);
-	return;
+	xfree_pptr(session.execargs);
 }
 
 void
@@ -84,27 +64,15 @@ quit(int code)
 	memset(session.mta_username, 0, sizeof(session.mta_username));
 	memset(session.mta_password, 0, sizeof(session.mta_password));
 
-	#ifdef HAVE_TLS
-	end_tls(CLIENT, secure.client);
-	end_tls(SERVER, secure.server);
-	if (secure.xcred)
-		gnutls_certificate_free_credentials(secure.xcred);
-	if (secure.x509_cred)
-		gnutls_certificate_free_credentials(secure.x509_cred);
-	gnutls_global_deinit();
-	#endif /* HAVE_TLS */
-
-	#ifdef HAVE_SSL
-	end_ssl(CLIENT, secure.client, secure.ctx_client);
-	end_ssl(SERVER, secure.server, secure.ctx_server);
-	#endif /* HAVE_SSL */
-
-	#ifdef HAVE_SYSLOG
+#ifdef USE_SSL
+	net_close(CLIENT, secure.client);
+	net_close(SERVER, secure.server);
+#endif
+#ifdef HAVE_SYSLOG
 	if ((topt & T_DAEMON) && !(topt & T_FOREGROUND))
 		closelog();
-	#endif /* HAVE_SYSLOG */
+#endif /* HAVE_SYSLOG */
 
-	close_rcfile();
 	free_mem();
 	exit(code);
 }
